# QDuckDB SqlNotebook Demo

If you want help there are guides located at http://www.timestored.com/qstudio/help/
Any feature requests etc feel free to raise on [github](https://github.com/timeseries/qstudio/issues).

[QDuckDB](https://www.timestored.com/qstudio/help/qduckdb) has the full power of DuckDB but with BabelDB placed ontop.
[BabelDB](https://www.timestored.com/pulse/help/babeldb) adds the ability to join data from different data sources and to save any table result locally
The examples below mostly use ``query_data`` to hardcode CSV tables as part of the query.

# Table

Table display can be configured using column names. See help->charts for details on format.


```sql type="grid" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Bar

- The first string columns are used as category labels.
- Whatever numeric columns appear after the strings represents a separate series in the chart.

```sql type="bar" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Bar Horizontal

```sql type="bar_horizontal" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Stack

- The first string columns are used as category labels.
- Whatever numeric columns appear after the strings represents a separate series in the chart.

```sql type="stack" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Bar Horizontal

```sql type="stack_horizontal" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Line

- The first string columns are used as category labels.
- Whatever numeric columns appear after the strings represents a separate series in the chart.

```sql type="line" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```


# Area

- The first string columns are used as category labels.
- Whatever numeric columns appear after the strings represents a separate series in the chart.
    
```sql type="area" server="QDUCKDB"
query_data('NAME, QUANTITY, MID
	OKTA, 100, 249.30
	TWLO, 10, 593.4
	TTD, 50, 560.3
	DOCU, 500, 244.0
	NFLX, 233, 676.9
	AMZN, 432, 108.7
	TSLA, 270, 690.7
	AAPL, 322, 112.3
	BABA, 287, 266.1
	JPM, 127, 223.2
	JNJ, 123, 87.8
	FB, 312, 308.7
	MSFT, 165, 379.1')
```

# Pie

 - Each numeric column represents one pie chart. The title of each pie chart will be the column title.
 - The segments of the pie chart use the string columns as a title where possible. If there are no string columns, row numbers are used.

```sql type="pie" server="QDUCKDB"
query_data('NAME	QUANTITY
 GOOG	654
 DOCU	500
 AMZN	432
 AAPL	322
 FB	312
 BABA	287
 TSLA	270')
```

# TreeMap

 - Starting from the left each string column is taken as one nesting level
 - The first numerical column will be taken as size.

```sql type="treemap" server="QDUCKDB". 
select Bank,"Assets ($mil.)" as Millions from query_web('https://en.wikipedia.org/wiki/List_of_bank_failures_in_the_United_States_(2008%E2%80%93present)')
```


## Sankey

 - Assuming string columns named S1,S2,S3 with a numeric column of value V.
 - Each row represents one flow from the top level S1 to the leaf node S3. S1->S2->S3->V
 - The first numeric column reprents the size of the flow between nodes.
 - Sizes are back-propagated to the top level.
 - Null can be used to represent either gaps or allow assigning value to a node that is neither an inflow nor outflow.


```sql type="sankey" server="QDUCKDB"
query_data('OrderOrigin	Exchange	State	v
Internal	ICE	Partial	2
GUI	ICE		6
Web	ICE		8
Platform	NYSE	Partial	19
Internal	NYSE	Filled	16
GUI	NYSE		17
Web	LDN	New	10
Platform	LDN		11
Internal	LDN	Partial	5
GUI	CBE	New	13
Web	CBE	New	16
Platform	CBE	Filled	17
')
```

## Boxplot

 - Each numerical column in the table becomes one boxplot item in the chart.
 - The min/max/median/Q1/Q3 are calculated from the raw data.
 - This is inefficient as a lot more data is being passed than needed but useful for toggling an existing data set view quickly.

```sql type="boxplot" server="QDUCKDB"
QUERY_DATA('a,b,c
1,2,3
3,4,7
3,3,10')
```

## Metrics

 - Two or more numeric columns are required.
 - The values in the first column are used for the X-axis.
 - The values in following columns are used for the Y-axis. Each column is displayed with a separate color.


```sql type="metrics" server="QDUCKDB"
QUERY_DATA('a,b,c
1,2,3')
```
